/*
 *
 * This program is written by
 * Vincent Guerin
 * and Ludovic Rousseau <ludovic.rousseau@gemplus.com>
 *
 * $Id: cmkdir_SC.c,v 1.4 2002/03/12 09:36:17 rousseau Exp $
 *
 */

/*
 * Copyright  2000 Gemplus
 * Av. du Pic de Bertagne - Parc d'Activits de Gmenos
 * BP 100 - 13881 Gmenos CEDEX
 * 
 * Everyone is allowed to redistribute and use this source  (source
 * code)  and binary (object code),  with or  without modification,
 * under some conditions:
 * 
 *  - Everyone  must  retain  and/or  reproduce the above copyright
 *    notice,  and the below  disclaimer of warranty and limitation
 *    of liability  for redistribution and use of these source code
 *    and object code.
 * 
 *  - Everyone  must  ask a  specific prior written permission from
 *    Gemplus to use the name of Gemplus.
 * 
 * DISCLAIMER OF WARRANTY
 * 
 * THIS CODE IS PROVIDED "AS IS",  WITHOUT ANY WARRANTY OF ANY KIND
 * (INCLUDING,  BUT  NOT  LIMITED  TO,  THE IMPLIED  WARRANTIES  OF
 * MERCHANTABILITY  AND FITNESS FOR  A  PARTICULAR PURPOSE)  EITHER
 * EXPRESS OR IMPLIED.  GEMPLUS DOES NOT WARRANT THAT THE FUNCTIONS
 * CONTAINED  IN THIS SOFTWARE WILL MEET THE USER'S REQUIREMENTS OR
 * THAT THE OPERATION OF IT WILL BE UNINTERRUPTED OR ERROR-FREE. NO
 * USE  OF  ANY  CODE  IS  AUTHORIZED  HEREUNDER EXCEPT UNDER  THIS
 * DISCLAIMER.
 * 
 * LIMITATION OF LIABILITY
 * 
 * GEMPLUS SHALL NOT BE LIABLE FOR INFRINGEMENTS OF  THIRD  PARTIES
 * RIGHTS. IN NO EVENTS, UNLESS REQUIRED BY APPLICABLE  LAW,  SHALL
 * GEMPLUS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES OF ANY CHARACTER  INCLUDING,
 * WITHOUT LIMITATION, DAMAGES FOR LOSS OF GOODWILL, WORK STOPPAGE,
 * COMPUTER FAILURE OR MALFUNCTION, OR ANY AND ALL OTHER DAMAGES OR
 * LOSSES, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. ALSO,
 * GEMPLUS IS  UNDER NO  OBLIGATION TO MAINTAIN,  CORRECT,  UPDATE,
 * CHANGE, MODIFY, OR OTHERWISE SUPPORT THIS SOFTWARE.
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>
#include <gmp.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/wait.h>
#include <sys/stat.h>
#include <gmp.h>

#include "config.h"
#include "global.h"
#include "smartcard_functions.h"
#include "cfs_SC_functions.h"

// cmkdir_SC is a wrapper for the cfs function cmkdir in order to
// use smartcards to store the passphrase.
// It also uses a file: the user cfs fstab file which put in correspondence
// ciphered directories and their clear name with a supplementary
// flag indicating if the user want its directories be auto attached.
// The new created directory has a default entry in this file.
// So, this wrapper uses and manages this files, and call the cattach binary.
//
// This binary takes the sames arguments as cmkdir excepted the -- option.
// User must amso provide a ciphered directory name.
/////////////////////////////////////////////////////////////////////////////

// Internal functions
/////////////////////

// Prints the program usage informations
void usage(char *progName);

// Cipher and saved the CFS secret in the directory
int save_secret(unsigned char *cfsSecret, char *directory);

int main(int argc, char *argv[])
{
	int rCode, result;
	int pid, status;
	int fildes[2];
	char *pinReaded;
	char *cipheredDirName;
	char pin[USR_PIN_SIZE + 1];
	unsigned char cfsSecret[CFS_SECRET_SIZE + 1];

	// Get the arguments
	// We just need the last arg, 
	// which it's supposed to be the directory name.
	////////////////////////////////////////////////

	if (argc < 2)
		usage(argv[0]);

	cipheredDirName = argv[argc - 1];

	// Get the user PIN using getpass to secure the transaction
	pinReaded = getpass("Enter the your pin: ");
	strncpy(pin, pinReaded, USR_PIN_SIZE);
	pin[USR_PIN_SIZE] = '\0';

	// Get the CFS secret stored in the card
	rCode = get_cfs_secret(pin, cfsSecret, 0);

	cfsSecret[CFS_SECRET_SIZE] = '\n';

	if (rCode == ERROR_CARD_NOT_FORMATED)
	{
		fprintf(stderr, "ERROR: this CARD seems to be NOT FORMATTED!\n");
		exit(EXIT_FAILURE);
	}

	if (rCode == ERROR_VERIFY_PIN)
	{
		// Specific wrong PIN code error
		fprintf(stderr, "ERROR: user PIN code is WRONG!\n");
		exit(EXIT_FAILURE);
	}

	if (rCode == EXIT_FAILURE)
	{
		// Other error
		fprintf(stderr, "UNABLE to get the CFS secret from the card\n");
		exit(EXIT_FAILURE);
	}

	// Create a pipe to send the CFS secret to cmkdir
	rCode = pipe(fildes);

	if (rCode == -1)
	{
		// Pipe can't be created
		fprintf(stderr, "UNABLE to create the pipe: %s\n", strerror(errno));
		exit(EXIT_FAILURE);
	}

	// We have to fork before execute the cmkdir binary
	pid = fork();

	if (pid == -1)
	{
		// Unable to fork
		fprintf(stderr, "UNABLE to fork: %s\n", strerror(errno));
		exit(EXIT_FAILURE);
	}

	if (pid == 0)
	{
		// We are into the child process
		//
		// Prepare the environment before the call of the cmkdir binary
		///////////////////////////////////////////////////////////////

		int i;
		char **argvTmp;

		// We want to use the -- option; with such option cmkdir will read
		// the CFS secret on stdin. So if we want father to directly send the 
		// secret to his child, we have to dup stdin to the reading part of 
		// the pipe.
		/////////////////////////////////////////////////////////////////////

		rCode = dup2(fildes[0], STDIN_FILENO);

		if (rCode == -1)
		{
			// UNABLE to dup... 
			fprintf(stderr, "UNABLE to dup stdin on the pipe: %s\n",
				strerror(errno));
			exit(EXIT_FAILURE);
		}

		// Reconstruct an arguments array
		/////////////////////////////////

		// Allocate a new array
		argvTmp = calloc(argc + 2, sizeof(char *));

		// Add the command name      
		argvTmp[0] = CMKDIR;

		// Add the -- option
		// cmkdir(1): The -- option will read the key from standard input
		// So cattach will read the key from the pipe on which the father
		// will send the key...
		//////////////////////////////////////////////////////////////////      

		argvTmp[1] = "--";

		// Store the other parameters
		for (i = 1; i < argc; i++)
			argvTmp[i + 1] = argv[i];

		// Add the null argument to the end of the tab
		argv[argc + 1] = NULL;

		// Execute cmkdir binary
		rCode = execvp((CFS_BIN_DIR CMKDIR), argvTmp);

		if (rCode == -1)
		{
			// Can't execute cmkdir
			fprintf(stderr, "UNABLE to execute %s: %s\n",
				CMKDIR, strerror(errno));
			exit(EXIT_FAILURE);
		}
	}

	else
	{
		// We are still into the father process
		///////////////////////////////////////

		// Write the CFS secret on the pipe
		rCode = write(fildes[1], cfsSecret, CFS_SECRET_SIZE + 1);

		if (rCode != CFS_SECRET_SIZE + 1)
		{
			// Can't write in the pipe
			fprintf(stderr,
				"UNABLE to write the CFS secret on the pipe: %s\n",
				strerror(errno));
			exit(EXIT_FAILURE);
		}

		// Wait for the end of the child
		rCode = waitpid(pid, &status, 0);

		if (WEXITSTATUS(status) != 0)
		{
			// The child (cmkdir) has returned with an error code
			fprintf(stderr, "UNABLE to create the ciphered directory %s\n",
				cipheredDirName);
			exit(EXIT_FAILURE);
		}
	}

	// Cipher the CFS secret and saved it in the directory
	rCode = save_secret(cfsSecret, cipheredDirName);

	if (rCode == EXIT_FAILURE)
	{
		// Can't saved the ciphered CFS secret
		fprintf(stderr, "UNABLE to save the ciphered CFS secret\n");
		fprintf(stderr,
			"You won't be able to restore your data if you loose your card\n");
	}

	// Now we want to add this new created directory in 
	// the user cfs fstab file.
	///////////////////////////////////////////////////

	// Test if the user cfs fstab file exists
	rCode = is_cfs_fstab_exist(&result, 0);

	if (rCode == EXIT_FAILURE)
	{
		// Can't do the test
		fprintf(stderr,
			"UNABLE to test the existence of the user cfs fstab file\n");
		exit(EXIT_FAILURE);
	}

	if (!result)
	{
		// The user cfs fstab file doesn't exist so create it.
		rCode = create_cfs_fstab(0);

		if (rCode == EXIT_FAILURE)
		{
			// Can't create the file
			fprintf(stderr, "UNABLE to create the user cfs fstab file\n");
			exit(EXIT_FAILURE);
		}
	}

	// Add this new entry in the cfs fstab file.
	// By default associated clear name is the ciphered directory name
	// and auto attach flag is set to noauto.
	//////////////////////////////////////////////////////////////////

	rCode = add_cfs_fstab_entry(cipheredDirName, 0);

	if (rCode == EXIT_FAILURE)
	{
		// Can't update the file
		fprintf(stderr,
			"UNABLE to add this new directory in the user cfs fstab file\n");
		exit(EXIT_FAILURE);
	}

	return (EXIT_SUCCESS);
}

void usage(char *progName)
{
	// Prints the program usage informations
	////////////////////////////////////////

	fprintf(stderr, "Usage: %s directory\n", progName);
	exit(EXIT_FAILURE);
}

int save_secret(unsigned char *cfsSecret, char *directory)
{
	int i, rCode;
	int ret = EXIT_SUCCESS;
	FILE *fd;
	mpz_t mpzCertExpo;
	mpz_t mpzCertModulo;
	mpz_t mpzCfsSecret;
	mpz_t mpzCipheredCfsSecret;
	char tmp[FILENAME_MAX];
	char cfsSecretTxt[CFS_SECRET_SIZE * 2 + 1];
	char certKeyTxt[REST_RSA_KEY_SIZE_IN_TXT + 1];

	// Get the administrator restauration public key
	////////////////////////////////////////////////

	// Open the root restoration key file
	fd = fopen(ROOT_REST_FILE_NAME, "r");

	if (fd == NULL)
	{
		fprintf(stderr,
			"UNABLE to open the administrator restauration public key file (%s): %s\n",
			ROOT_REST_FILE_NAME, strerror(errno));
		return (EXIT_FAILURE);
	}

	// Read the root public key
	for (i = 0; i < REST_RSA_KEY_SIZE_IN_TXT; i++)
	{
		rCode = fscanf(fd, "%c", certKeyTxt + i);

		if (rCode != 1)
		{
			fprintf(stderr,
				"UNABLE to read the root restauartion public key: %s\n",
				strerror(errno));
			return (EXIT_FAILURE);
		}
	}

	fclose(fd);

	// Convert the public modulo and the public exposant in a mpz number
	////////////////////////////////////////////////////////////////////

	// Convert the CFS secret in a byte form

	for (i = 0; i < CFS_SECRET_SIZE; i++)
		sprintf(cfsSecretTxt + i * 2, "%02x", cfsSecret[i]);

	certKeyTxt[REST_RSA_KEY_SIZE_IN_TXT] = '\0';
	cfsSecretTxt[CFS_SECRET_SIZE * 2] = '\0';

	mpz_init(mpzCertExpo);
	mpz_init(mpzCertModulo);
	mpz_init(mpzCfsSecret);
	mpz_init(mpzCipheredCfsSecret);

	mpz_init_set_str(mpzCertExpo, RSA_PUBLIC_EXPOSANT, 16);
	mpz_init_set_str(mpzCertModulo, certKeyTxt, 16);
	mpz_init_set_str(mpzCfsSecret, cfsSecretTxt, 16);

	// Encrypt the CFS secret with the administrator restore CFS 
	// public key. Just here is the operation which perform the
	// RSA ciphering
	////////////////////////////////////////////////////////////

	mpz_powm(mpzCipheredCfsSecret, mpzCfsSecret, mpzCertExpo, mpzCertModulo);

	// Store the ciphered cfs secret.
	/////////////////////////////////

	// Construct the file name
	sprintf(tmp, "%s/%s", directory, SAVED_SECRET_FILE);

	// Open the file
	fd = fopen(tmp, "w");

	if (fd == NULL)
	{
		fprintf(stderr,
			"UNABLE to open the ciphered CFS secret file (%s): %s\n",
			tmp, strerror(errno));
		ret = EXIT_FAILURE;
		goto clean_and_return;
	}

	// Write the ciphered CFS secret
	mpz_out_str(fd, 16, mpzCipheredCfsSecret);

	fprintf(fd, "\n");

	fclose(fd);

	// Change the file acces attribut
	// The file is only readable by it's owner
	//////////////////////////////////////////

	rCode = chmod(tmp, S_IRUSR);

	if (rCode == -1)
	{
		fprintf(stderr,
			"UNABLE to change the attributs of the ciphered CFS secret file (%s): %s\n",
			tmp, strerror(errno));
		ret = EXIT_FAILURE;
	}

clean_and_return:
	mpz_clear(mpzCertExpo);
	mpz_clear(mpzCertModulo);
	mpz_clear(mpzCfsSecret);
	mpz_clear(mpzCipheredCfsSecret);

	return ret;
}

