/*
 *
 * This program is written by
 * Vincent Guerin
 * and Ludovic Rousseau <ludovic.rousseau@gemplus.com>
 *
 * $Id: crescue_SC.c,v 1.2 2001/01/03 09:55:55 rousseau Exp $
 *
 */

/*
 * Copyright  2000 Gemplus
 * Av. du Pic de Bertagne - Parc d'Activits de Gmenos
 * BP 100 - 13881 Gmenos CEDEX
 * 
 * Everyone is allowed to redistribute and use this source  (source
 * code)  and binary (object code),  with or  without modification,
 * under some conditions:
 * 
 *  - Everyone  must  retain  and/or  reproduce the above copyright
 *    notice,  and the below  disclaimer of warranty and limitation
 *    of liability  for redistribution and use of these source code
 *    and object code.
 * 
 *  - Everyone  must  ask a  specific prior written permission from
 *    Gemplus to use the name of Gemplus.
 * 
 * DISCLAIMER OF WARRANTY
 * 
 * THIS CODE IS PROVIDED "AS IS",  WITHOUT ANY WARRANTY OF ANY KIND
 * (INCLUDING,  BUT  NOT  LIMITED  TO,  THE IMPLIED  WARRANTIES  OF
 * MERCHANTABILITY  AND FITNESS FOR  A  PARTICULAR PURPOSE)  EITHER
 * EXPRESS OR IMPLIED.  GEMPLUS DOES NOT WARRANT THAT THE FUNCTIONS
 * CONTAINED  IN THIS SOFTWARE WILL MEET THE USER'S REQUIREMENTS OR
 * THAT THE OPERATION OF IT WILL BE UNINTERRUPTED OR ERROR-FREE. NO
 * USE  OF  ANY  CODE  IS  AUTHORIZED  HEREUNDER EXCEPT UNDER  THIS
 * DISCLAIMER.
 * 
 * LIMITATION OF LIABILITY
 * 
 * GEMPLUS SHALL NOT BE LIABLE FOR INFRINGEMENTS OF  THIRD  PARTIES
 * RIGHTS. IN NO EVENTS, UNLESS REQUIRED BY APPLICABLE  LAW,  SHALL
 * GEMPLUS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES OF ANY CHARACTER  INCLUDING,
 * WITHOUT LIMITATION, DAMAGES FOR LOSS OF GOODWILL, WORK STOPPAGE,
 * COMPUTER FAILURE OR MALFUNCTION, OR ANY AND ALL OTHER DAMAGES OR
 * LOSSES, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. ALSO,
 * GEMPLUS IS  UNDER NO  OBLIGATION TO MAINTAIN,  CORRECT,  UPDATE,
 * CHANGE, MODIFY, OR OTHERWISE SUPPORT THIS SOFTWARE.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/stat.h>
#include <fcntl.h>

#include "config.h"
#include "global.h"
#include "smartcard_functions.h"

// Print program usage informations
void usage(char *progName);

int main(int argc, char **argv)
{
	// This function restores the CFS secret used to create the 
	// ciphered directory. The ciphered CFS secret is stored in
	// .saved_secret in the directory. It is unciphered using
	// the root certification card. The unciphered CFS secret is
	// stored in restored_secret.
	////////////////////////////////////////////////////////////


	int temp;
	int i, desc;
	int rCode;
	int ownerUid;
	FILE *fd;
	struct stat finfo;
	char *cipheredDirName = NULL;
	char *passphraseReaded;
	char cipheredSecretName[FILENAME_MAX];
	char passphrase[PASSPHRASE_MAX_LEN + 1];
	unsigned char cfsSecret[CFS_SECRET_SIZE];
	unsigned char cipheredCfsSecret[CFS_SECRET_SIZE];

	// Get the argument
	///////////////////

	if (argc != 2)
		usage(argv[0]);
	else
		cipheredDirName = argv[1];

	// Get the owner uid of the ciphered CFS secret file
	////////////////////////////////////////////////////

	// Construct the ciphered CFS secret file name
	sprintf(cipheredSecretName, "%s/%s", cipheredDirName, SAVED_SECRET_FILE);

	// Get the owner uid
	rCode = stat(cipheredSecretName, &finfo);

	ownerUid = finfo.st_uid;

	if (rCode == -1)
	{
		fprintf(stderr,
			"UNABLE to stat the ciphered CFS secret file (%s): %s\n",
			cipheredSecretName, strerror(errno));
		return (EXIT_FAILURE);
	}

	// Test the process uid
	if ((getuid() != 0) && (getuid() != ownerUid))
	{
		fprintf(stderr,
			"You must be root or the owner of the ciphered directory\n");
		exit(EXIT_FAILURE);
	}

	// If the process owner is root, change it to the 
	// owner of the ciphered directory
	/////////////////////////////////////////////////

	if (getuid() != 0)
	{
		rCode = setuid(ownerUid);

		if (rCode == -1)
		{
			fprintf(stderr, "UNABLE to set the process uid to the uid of\n");
			fprintf(stderr, "owner of the ciphered directory: %s",
				strerror(errno));
			exit(EXIT_FAILURE);
		}
	}

	fprintf(stdout, "Insert the administrator certification smartcard\n");

	// Get the administrator passphrase
	///////////////////////////////////

	passphraseReaded = getpass("Enter your passphrase: ");
	strncpy(passphrase, passphraseReaded, PASSPHRASE_MAX_LEN);
	passphrase[PASSPHRASE_MAX_LEN] = '\0';

	// Read the ciphered CFS secret
	///////////////////////////////

	// Open the ciphered CFS secret file
	fd = fopen(cipheredSecretName, "r");

	if (fd == NULL)
	{
		fprintf(stderr,
			"UNABLE to open the ciphered CFS secret file (%s): %s\n",
			cipheredSecretName, strerror(errno));
		return (EXIT_FAILURE);
	}

	// Read the ciphered CFS secret
	for (i = 0; i < CFS_SECRET_SIZE; i++)
	{
		rCode = fscanf(fd, "%02x", &temp);

		if (rCode != 1)
		{
			fprintf(stderr,
				"UNABLE to read the ciphered CFS secret: %s\n",
				strerror(errno));
			return (EXIT_FAILURE);
		}
		else
			cipheredCfsSecret[i] = (unsigned char) temp;
	}

	fclose(fd);

	// Call the restore function
	////////////////////////////

	rCode = restore_cfs_secret(passphrase, strlen(passphrase),
		cipheredCfsSecret, cfsSecret);

	if (rCode == ERROR_CARD_NOT_FORMATED)
	{
		// This error is probably caused by a card on which a certification
		// part hasn't been created.
		fprintf(stderr,
			"ERROR: this CARD seems to be NOT A CERTIFICATION CARD!\n");
		return (EXIT_FAILURE);
	}

	if (rCode == ERROR_VERIFY_PIN)
	{
		// The administrator PUC is false, it's probably 
		// caused by a wrong passphrase.  
		fprintf(stderr,
			"ERROR: administrator PUC is FALSE: incorect passphrase!\n");
		return (EXIT_FAILURE);
	}

	if (rCode == EXIT_FAILURE)
	{
		fprintf(stderr, "UNABLE to restore the CFS secret\n");
		return (EXIT_FAILURE);
	}

	// Write the restored CFS secret in the restored_secret file
	////////////////////////////////////////////////////////////

	// Open the file
	desc = open(RESTORED_SECRET_FILE, O_WRONLY | O_CREAT);

	if (desc == -1)
	{
		fprintf(stderr,
			"UNABLE to open the restored CFS secret file (%s): %s\n",
			RESTORED_SECRET_FILE, strerror(errno));
		return (EXIT_FAILURE);
	}

	// Write the retored CFS secret
	rCode = write(desc, cfsSecret, CFS_SECRET_SIZE);

	if (rCode != CFS_SECRET_SIZE)
	{
		fprintf(stderr,
			"UNABLE to write the restored CFS secret file: %s\n",
			strerror(errno));
		return (EXIT_FAILURE);
	}

	close(desc);

	// Change the restored secret file owner to the owner of the 
	// ciphered CFS secret file
	////////////////////////////////////////////////////////////

	rCode = chown(RESTORED_SECRET_FILE, ownerUid, -1);

	if (rCode == -1)
	{
		fprintf(stderr,
			"UNABLE to change the owner of the restored CFS secret file (%s): %s\n",
			RESTORED_SECRET_FILE, strerror(errno));
		return (EXIT_FAILURE);
	}

	// Change the file acces attributs, the restored CFS secret file 
	// is only readable and writable by it's owner
	rCode = chmod(RESTORED_SECRET_FILE, S_IRUSR | S_IWRITE);

	if (rCode == -1)
	{
		fprintf(stderr,
			"UNABLE to change the attributs of the restored CFS secret file (%s): %s\n",
			RESTORED_SECRET_FILE, strerror(errno));
		return (EXIT_FAILURE);
	}

	return (EXIT_SUCCESS);
}

void usage(char *progName)
{
	// Print program usage informations
	///////////////////////////////////

	fprintf(stderr, "Usage: %s directory\n", progName);
	exit(EXIT_FAILURE);
}
