/*
 *
 * This program is written by
 * Vincent Guerin
 * and Ludovic Rousseau <ludovic.rousseau@gemplus.com>
 *
 * $Id: callbacks.c,v 1.5 2002/03/11 15:04:53 rousseau Exp $
 *
 */

/*
 * Copyright  2000 Gemplus
 * Av. du Pic de Bertagne - Parc d'Activits de Gmenos
 * BP 100 - 13881 Gmenos CEDEX
 * 
 * Everyone is allowed to redistribute and use this source  (source
 * code)  and binary (object code),  with or  without modification,
 * under some conditions:
 * 
 *  - Everyone  must  retain  and/or  reproduce the above copyright
 *    notice,  and the below  disclaimer of warranty and limitation
 *    of liability  for redistribution and use of these source code
 *    and object code.
 * 
 *  - Everyone  must  ask a  specific prior written permission from
 *    Gemplus to use the name of Gemplus.
 * 
 * DISCLAIMER OF WARRANTY
 * 
 * THIS CODE IS PROVIDED "AS IS",  WITHOUT ANY WARRANTY OF ANY KIND
 * (INCLUDING,  BUT  NOT  LIMITED  TO,  THE IMPLIED  WARRANTIES  OF
 * MERCHANTABILITY  AND FITNESS FOR  A  PARTICULAR PURPOSE)  EITHER
 * EXPRESS OR IMPLIED.  GEMPLUS DOES NOT WARRANT THAT THE FUNCTIONS
 * CONTAINED  IN THIS SOFTWARE WILL MEET THE USER'S REQUIREMENTS OR
 * THAT THE OPERATION OF IT WILL BE UNINTERRUPTED OR ERROR-FREE. NO
 * USE  OF  ANY  CODE  IS  AUTHORIZED  HEREUNDER EXCEPT UNDER  THIS
 * DISCLAIMER.
 * 
 * LIMITATION OF LIABILITY
 * 
 * GEMPLUS SHALL NOT BE LIABLE FOR INFRINGEMENTS OF  THIRD  PARTIES
 * RIGHTS. IN NO EVENTS, UNLESS REQUIRED BY APPLICABLE  LAW,  SHALL
 * GEMPLUS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES OF ANY CHARACTER  INCLUDING,
 * WITHOUT LIMITATION, DAMAGES FOR LOSS OF GOODWILL, WORK STOPPAGE,
 * COMPUTER FAILURE OR MALFUNCTION, OR ANY AND ALL OTHER DAMAGES OR
 * LOSSES, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. ALSO,
 * GEMPLUS IS  UNDER NO  OBLIGATION TO MAINTAIN,  CORRECT,  UPDATE,
 * CHANGE, MODIFY, OR OTHERWISE SUPPORT THIS SOFTWARE.
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <gtk/gtk.h>
#include <pwd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <errno.h>
#include <unistd.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"

#include "config.h"
#include "global.h"
#include "publickey_db_functions.h"
#include "smartcard_functions.h"

// Internal function used to update the registred users list
void updateList();

void on_button_format_clicked(GtkButton * button, gpointer user_data)
{
	// Format a new card
	////////////////////

	int rCode;

	rCode = formatcard(passphrase, passphraseLen);

	if (rCode == ERROR_CARD_ALREADY_FORMATED)
		create_window_dialog
			("ERROR: \nThis card seems to be ALREADY FORMATED");

	else if (rCode == EXIT_FAILURE)
		create_window_dialog("UNABLE to format this card");

	else
		create_window_dialog("Format card:\n SUCCESS");
}

void on_button_change_pin_clicked(GtkButton * button, gpointer user_data)
{
	// Change PIN or Unlock card
	////////////////////////////

	int rCode;
	gchar *pin1, *pin2;
	guint pinLength;
	char bufTmp[BUF_TMP_SIZE];

	pin1 = gtk_entry_get_text((GtkEntry *) entry7);
	pin2 = gtk_entry_get_text((GtkEntry *) entry9);

	pinLength = (int) ((GtkEntry *) entry7)->text_length;

	if (strcmp((char *) pin1, (char *) pin2) != 0)
	{
		create_window_dialog("PIN codes DONT'T MATCH");
		return;
	}

	if (pinLength == 0)			// No code given
	{
		create_window_dialog("New PIN REQUIRED");
		return;
	}

	if (pinLength != USR_PIN_SIZE)	// Code given have not the required size
	{
		sprintf(bufTmp, "Incorect PIN size \n %d characters REQUIRED",
			USR_PIN_SIZE);
		create_window_dialog(bufTmp);
		return;
	}

	rCode = pin_mngmt(passphrase, passphraseLen, (char *) pin1);

	if (rCode == ERROR_CARD_NOT_FORMATED)
		create_window_dialog("ERROR: \nThis card seems to be NOT FORMATED");

	else if (rCode == ERROR_VERIFY_PIN)
		create_window_dialog
			("ERROR: \nAdministrator PUC is FALSE:\nincorrect passphrase");

	else if (rCode == EXIT_FAILURE)
		create_window_dialog("UNABLE to unlock or change the PIN code");

	else
		create_window_dialog("Unlock card / Change PIN:\n SUCCESS");
}

void on_button_create_user_clicked(GtkButton * button, gpointer user_data)
{
	// Beginning of the create user mechanism
	/////////////////////////////////////////

	int rCode;
	gchar *usernamereaded;
	guint usernameLength;
	char bufTmp[BUF_TMP_SIZE + USER_NAME_MAX_SIZE];

	usernameLength = (int) ((GtkEntry *) entry8)->text_length;
	usernamereaded = gtk_entry_get_text((GtkEntry *) entry8);

	username = (char *) malloc(usernameLength * sizeof(char) + 1);
	strncpy(username, (char *) usernamereaded, usernameLength);
	username[usernameLength] = '\0';

	if (strlen(username) == 0)	// No username given
	{
		create_window_dialog("User name MUST BE DEFINED");
		return;
	}

	if (strlen(username) > USER_NAME_MAX_SIZE - 1)	// user name given too long
	{
		sprintf(bufTmp, "User name TOO WIDE, %d characters maximum",
			USER_NAME_MAX_SIZE - 1);
		create_window_dialog(bufTmp);
		return;
	}

	// pad username with \0
	{
		char *c;

		c = username + strlen(username);
		while (c < username + USER_NAME_MAX_SIZE)
		{
			*(c++) = '\0';
		}
	}

	rCode = perso_card(passphrase, passphraseLen, (char *) username,
		dbFileName, publicKey);

	if (rCode == ERROR_CARD_NOT_FORMATED)
	{
		create_window_dialog("ERROR: \nThis card seems to be NOT FORMATED");
		return;
	}

	if (rCode == ERROR_VERIFY_PIN)
	{
		create_window_dialog
			("ERROR: \nAdministrator PUC is FALSE:\nincorrect passphrase");
		return;
	}

	if (rCode == EXIT_FAILURE)
	{
		create_window_dialog("Unable to create this new user");
		return;
	}

	if (!twoReaders)
	{
		create_window_choice(CREATE_NEXT,
			"Please insert the certification card\nOK when it's done", "OK",
			"Cancel");
		return;
	}

	rCode = sign_user_public_key(publicKey, username, passphrase, signature, 1);

	if (rCode == ERROR_CARD_NOT_FORMATED)
	{
		// This error is probably caused by a card on which a certification
		// part hasn't been created.
		create_window_dialog
			("ERROR: this CARD seems to be\nNOT A CERTIFICATION CARD!\n");
		return;
	}

	if (rCode == ERROR_VERIFY_PIN)
	{
		create_window_dialog
			("ERROR: \nAdministrator PUC is FALSE:\nincorrect passphrase");
		return;
	}

	if (rCode == EXIT_FAILURE)
	{
		create_window_dialog
			("UNABLE to sign the public key\n of this new user");
		return;
	}

	rCode =
		publickey_db_add_user(username, publicKey, signature, dbFileName, 0);

	if (rCode == ERR_USR_ALREADY_IN_DB)
	{
		sprintf(bufTmp,
			"WARNING:\nUser %s ALREADY REGISTED in the database\nDo you want to REPLACE he?",
			username);
		create_window_choice(CREATE_USER, bufTmp, "Replace", "Cancel");

		return;
	}

	if (rCode == EXIT_FAILURE)
	{
		create_window_dialog
			("UNABLE to store the public key\nof this new user");
		return;
	}

	create_window_dialog("Create user: \n SUCCESS");

	gtk_entry_set_text(GTK_ENTRY(entry8), _(""));
	updateList();
}

void on_button_delete_user_clicked(GtkButton * button, gpointer user_data)
{
	// Delete user from the database
	////////////////////////////////

	gchar *usernameReaded;
	int rCode, usernameLength;
	char *username;
	char bufTmp[BUF_TMP_SIZE];

	usernameReaded = gtk_entry_get_text((GtkEntry *) entry8);
	usernameLength = (int) ((GtkEntry *) entry8)->text_length;

	username = (char *) malloc(usernameLength * sizeof(char) + 1);
	strncpy(username, (char *) usernameReaded, usernameLength);
	username[usernameLength] = '\0';

	if (strlen(username) == 0)
	{
		create_window_dialog("User name MUST BE DEFINED");
		return;
	}

	if (strlen(username) > USER_NAME_MAX_SIZE - 1)
	{
		sprintf(bufTmp, "User name TOO WIDE, %d characters maximum",
			USER_NAME_MAX_SIZE - 1);
		create_window_dialog(bufTmp);
		return;
	}

	rCode = publickey_db_delete_user(username, dbFileName);

	if (rCode == USR_NOT_IN_DB)
		create_window_dialog
			("ERROR: \nThis user is NOT REGISTRED in the database");

	else if (rCode == EXIT_FAILURE)
		create_window_dialog("UNABLE to delete this user");

	else
	{
		create_window_dialog("Delete user:\n SUCCESS");

		gtk_entry_set_text(GTK_ENTRY(entry8), _(""));
		updateList();
	}
}

void on_button_verify_clicked(GtkButton * button, gpointer user_data)
{
	// Verify that there is no public key repeated in the database
	// which probably means that administator have format a card 
	// for a new user without have deleted it old user in the database

	int size;
	char user_1[USER_NAME_MAX_SIZE];
	char user_2[USER_NAME_MAX_SIZE];
	char buffer[1000];
	struct passwd *passwordUser_1Entry = NULL;
	struct passwd *passwordUser_2Entry = NULL;
	char *user_1Info[1024], *user_2Info[1024];

	memset(buffer, '\0', 1000);

	size = publickey_db_verify_key(dbFileName, user_1, user_2);

	if (size == 0)
		sprintf(buffer, "NO PROBLEMS in public key database");

	else
	{
		passwordUser_1Entry = getpwnam(user_1);
		if (passwordUser_1Entry != NULL)
			strcpy((char *) user_1Info, passwordUser_1Entry->pw_gecos);
		else
			strcpy((char *) user_1Info, "No Info available");

		passwordUser_2Entry = getpwnam(user_2);
		if (passwordUser_2Entry != NULL)
			strcpy((char *) user_2Info, passwordUser_2Entry->pw_gecos);
		else
			strcpy((char *) user_2Info, "No Info available");

		sprintf(buffer,
			"These users have the SAME PUBLIC KEY:\n %s: %s \n %s: %s",
			(char *) user_1, (char *) user_1Info, (char *) user_2,
			(char *) user_2Info);
	}

	gtk_entry_set_text(GTK_ENTRY(entry8), _(""));
	create_window_dialog(buffer);
}

void on_button_quit_clicked(GtkButton * button, gpointer user_data)
{
	// Just kill the widwed and remove the lock file
	////////////////////////////////////////////////

	gtk_widget_destroy(gtk_widget_get_toplevel(GTK_WIDGET(button)));

	// Remove the lock file
	unlink(SST_LOCK_FILE);

	exit(EXIT_SUCCESS);
}

void on_button_create_cert_card_clicked(GtkButton * button,
	gpointer user_data)
{
	// Create an administrator certification card
	/////////////////////////////////////////////

	int rCode;

	rCode = generate_certification_part(passphrase, passphraseLen);

	if (rCode == ERROR_CARD_ALREADY_FORMATED)
		create_window_dialog
			("ERROR: \nThis card seems to be ALREADY a CERTIFICATION CARD");

	else if (rCode == EXIT_FAILURE)
		create_window_dialog("UNABLE to create the certification card");

	else
		create_window_dialog("Create certification card:\n SUCCESS");
}

void on_button_install_cert_key_clicked(GtkButton * button,
	gpointer user_data)
{
	// Install the administrator certification public key
	/////////////////////////////////////////////////////

	int rCode;

	rCode = install_certification_part(passphrase, passphraseLen);

	if (rCode == ERROR_CARD_NOT_FORMATED)
		create_window_dialog
			("ERROR: \nThis card seems NOT to be a CERTIFICATION CARD");

	else if (rCode == ERROR_VERIFY_PIN)
		create_window_dialog
			("ERROR: \nAdministrator PUC is FALSE:\nincorect passphrase");

	else if (rCode == EXIT_FAILURE)
		create_window_dialog("UNABLE to install the certification key");

	else
		create_window_dialog("Install certification key:\n SUCCESS");
}

void on_button_firstWindow_ok_clicked(GtkButton * button, gpointer user_data)
{
	int rCode;
	guchar *passphrase1;
	guchar *passphrase2;
	guchar *db_file;
	guint16 passphraseLength;
	guint16 dbLength;
	char bufTmp[BUF_TMP_SIZE + DB_NAME_MAX_LEN];

	passphrase1 = gtk_entry_get_text((GtkEntry *) entry10);
	passphrase2 = gtk_entry_get_text((GtkEntry *) entry11);
	db_file = gtk_entry_get_text((GtkEntry *) entry12);

	passphraseLength = (int) ((GtkEntry *) entry10)->text_length;
	dbLength = (int) ((GtkEntry *) entry12)->text_length;

	if (strcmp((char *) passphrase1, (char *) passphrase2) != 0)
	{
		create_window_dialog("Passphrases DON'T MATCH");
		return;
	}

	passphrase = (char *) malloc(passphraseLength * sizeof(char) + 1);
	strncpy(passphrase, (char *) passphrase1, passphraseLength);
	passphrase[passphraseLength] = '\0';

	passphraseLen = passphraseLength;

	if (passphraseLen < PASSPHRASE_MIN_LEN)
	{
		sprintf(bufTmp,
			"Passphrase TOO SHORT\n At least %d characters required",
			PASSPHRASE_MIN_LEN);
		create_window_dialog(bufTmp);
		return;
	}

	if (passphraseLen > PASSPHRASE_MAX_LEN)
	{
		sprintf(bufTmp, "Passphrase TOO WIDE, %d characters maximun",
			PASSPHRASE_MAX_LEN);
		create_window_dialog(bufTmp);
		return;
	}

	// Are there two readers?

	if (gtk_toggle_button_get_active((GtkToggleButton *) checkbutton1))
		twoReaders = 1;

	if (dbLength == 0)
	{
		create_window_dialog("You MUST ENTER the database name");
		return;
	}

	dbFileName = (char *) malloc(dbLength * sizeof(char) + 1);
	strncpy(dbFileName, (char *) db_file, dbLength);
	dbFileName[dbLength] = '\0';

	// If the database is the default database
	// Then Create it if it doesn't exist yet

	if (strcmp(dbFileName, DEFAULT_DB_NAME) == 0)
		public_key_db_create_db(DEFAULT_DB_NAME, 0);

	// Test the existence of database file

	rCode = test_db(dbFileName);

	if (rCode == EXIT_FAILURE)
	{
		sprintf(bufTmp,
			"WARNING: \n%s database DOESN'T EXIST \nDo you want to CREATE it?",
			dbFileName);
		create_window_choice(NEW_DB, bufTmp, "Create", "Cancel");
		return;
	}

	gtk_widget_hide(firstWindow);
	mainWindow = create_mainWindow();
}


void on_button_firstWindow_cancel_clicked(GtkButton * button,
	gpointer user_data)
{
	// First window canceled, so we can exit
	////////////////////////////////////////

	gtk_widget_destroy(gtk_widget_get_toplevel(GTK_WIDGET(button)));

	// Remove the lock file
	unlink(SST_LOCK_FILE);

	exit(EXIT_SUCCESS);
}

void on_window_dialog_ok_clicked(GtkButton * button, gpointer user_data)
{
	// Close the window dialog
	//////////////////////////

	gtk_widget_destroy(gtk_widget_get_toplevel(GTK_WIDGET(button)));
}

void on_window_choice_button1_clicked_for_create_user(GtkButton * button,
	gpointer user_data)
{
	// Replace the user in the database
	// with the forced flag set
	///////////////////////////////////

	int rCode;

	gtk_widget_destroy(gtk_widget_get_toplevel(GTK_WIDGET(button)));

	rCode =
		publickey_db_add_user(username, publicKey, signature, dbFileName, 1);

	if (rCode == EXIT_FAILURE)
		create_window_dialog("UNABLE to create this new user");

	else
		create_window_dialog("Create user:\nSUCCESS");
}

void on_window_choice_button2_clicked_for_create_user(GtkButton * button,
	gpointer user_data)
{
	// Nothing to do, close the window
	//////////////////////////////////

	gtk_widget_destroy(gtk_widget_get_toplevel(GTK_WIDGET(button)));
}

void on_window_choice_button1_clicked_for_new_db(GtkButton * button,
	gpointer user_data)
{
	// Create a new public key database
	///////////////////////////////////

	int rCode;

	gtk_widget_destroy(gtk_widget_get_toplevel(GTK_WIDGET(button)));

	rCode = public_key_db_create_db(dbFileName, 1);

	if (rCode == EXIT_FAILURE)
		create_window_dialog("UNABLE to create this database");

	else
	{
		create_window_dialog("Create new database:\n SUCCESS");
		gtk_widget_hide(firstWindow);
		mainWindow = create_mainWindow();
	}
}

void on_window_choice_button2_clicked_for_new_db(GtkButton * button,
	gpointer user_data)
{
	// Nothing to do, close the window
	//////////////////////////////////

	gtk_widget_destroy(gtk_widget_get_toplevel(GTK_WIDGET(button)));
}

void on_window_choice_button1_clicked_for_create_next(GtkButton * button,
	gpointer user_data)
{
	// Root certification card is now inserted in the reader
	// We can sign the user public key and write in the database
	////////////////////////////////////////////////////////////

	int rCode;
	char bufTmp[BUF_TMP_SIZE + USER_NAME_MAX_SIZE];

	gtk_widget_destroy(gtk_widget_get_toplevel(GTK_WIDGET(button)));

	rCode = sign_user_public_key(publicKey, username, passphrase, signature, 0);

	if (rCode == ERROR_CARD_NOT_FORMATED)
	{
		// This error is probably caused by a card on which a certification
		// part hasn't been created.
		create_window_dialog
			("ERROR: this CARD seems to be\nNOT A CERTIFICATION CARD!\n");
		return;
	}

	if (rCode == ERROR_VERIFY_PIN)
	{
		create_window_dialog
			("ERROR: \nAdministrator PUC is FALSE:\nincorrect passphrase");
		return;
	}

	if (rCode == EXIT_FAILURE)
	{
		create_window_dialog
			("UNABLE to sign the public key\n for this new user");
		return;
	}

	rCode =
		publickey_db_add_user(username, publicKey, signature, dbFileName, 0);

	if (rCode == ERR_USR_ALREADY_IN_DB)
	{
		sprintf(bufTmp,
			"WARNING:\nUser %s ALREADY REGISTRED in the database\nDo you want to REPLACE he?",
			username);
		create_window_choice(CREATE_USER, bufTmp, "Replace", "Cancel");

		return;
	}

	if (rCode == EXIT_FAILURE)
	{
		create_window_dialog
			("UNABLE to store the public key\n of this new user");
		return;
	}

	create_window_dialog("Create user:\n SUCCESS");

	gtk_entry_set_text(GTK_ENTRY(entry8), _(""));
	updateList();
}

void on_window_choice_button2_clicked_for_create_next(GtkButton * button,
	gpointer user_data)
{
	// End of create user procedure, close the window
	/////////////////////////////////////////////////

	gtk_widget_destroy(gtk_widget_get_toplevel(GTK_WIDGET(button)));
}

void on_master1_switch_page(GtkButton * button, gpointer user_data)
{
	// Just update the registred user list
	updateList();
}

void on_clist1_selection_made(GtkWidget * clist, gint row, gint column,
	GdkEventButton * event, gpointer data)
{
	// Print the list selected user in the user entry box
	/////////////////////////////////////////////////////

	gchar *text;

	gtk_clist_get_text(GTK_CLIST(clist1), row, column, &text);
	gtk_entry_set_text(GTK_ENTRY(entry8), _(text));
}

void updateList()
{
	// Print the list of selected database registred user 
	// in the list window
	/////////////////////////////////////////////////////

	int i, size, rCode;
	char **list;

	gtk_clist_clear(GTK_CLIST(clist1));

	rCode = publickey_db_list_user(dbFileName, &list, &size);

	if (rCode == EXIT_SUCCESS)
	{
		for (i = 0; i < size; i++)
		{
			gtk_clist_append(GTK_CLIST(clist1), (gchar **) (list + i));
			free(list[i]);
		}
		free(list);
	}

	else						// There is a problem...
		gtk_clist_append(GTK_CLIST(clist1),
			(gchar **) (&"No list available"));
}
