/*
 *
 * This program is written by
 * Vincent Guerin
 * and Ludovic Rousseau <ludovic.rousseau@gemplus.com>
 *
 * $Id: cfs_SC_functions.h,v 1.3 2002/03/07 10:33:05 rousseau Exp $
 *
 */

/*
 * Copyright  2000 Gemplus
 * Av. du Pic de Bertagne - Parc d'Activits de Gmenos
 * BP 100 - 13881 Gmenos CEDEX
 * 
 * Everyone is allowed to redistribute and use this source  (source
 * code)  and binary (object code),  with or  without modification,
 * under some conditions:
 * 
 *  - Everyone  must  retain  and/or  reproduce the above copyright
 *    notice,  and the below  disclaimer of warranty and limitation
 *    of liability  for redistribution and use of these source code
 *    and object code.
 * 
 *  - Everyone  must  ask a  specific prior written permission from
 *    Gemplus to use the name of Gemplus.
 * 
 * DISCLAIMER OF WARRANTY
 * 
 * THIS CODE IS PROVIDED "AS IS",  WITHOUT ANY WARRANTY OF ANY KIND
 * (INCLUDING,  BUT  NOT  LIMITED  TO,  THE IMPLIED  WARRANTIES  OF
 * MERCHANTABILITY  AND FITNESS FOR  A  PARTICULAR PURPOSE)  EITHER
 * EXPRESS OR IMPLIED.  GEMPLUS DOES NOT WARRANT THAT THE FUNCTIONS
 * CONTAINED  IN THIS SOFTWARE WILL MEET THE USER'S REQUIREMENTS OR
 * THAT THE OPERATION OF IT WILL BE UNINTERRUPTED OR ERROR-FREE. NO
 * USE  OF  ANY  CODE  IS  AUTHORIZED  HEREUNDER EXCEPT UNDER  THIS
 * DISCLAIMER.
 * 
 * LIMITATION OF LIABILITY
 * 
 * GEMPLUS SHALL NOT BE LIABLE FOR INFRINGEMENTS OF  THIRD  PARTIES
 * RIGHTS. IN NO EVENTS, UNLESS REQUIRED BY APPLICABLE  LAW,  SHALL
 * GEMPLUS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES OF ANY CHARACTER  INCLUDING,
 * WITHOUT LIMITATION, DAMAGES FOR LOSS OF GOODWILL, WORK STOPPAGE,
 * COMPUTER FAILURE OR MALFUNCTION, OR ANY AND ALL OTHER DAMAGES OR
 * LOSSES, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. ALSO,
 * GEMPLUS IS  UNDER NO  OBLIGATION TO MAINTAIN,  CORRECT,  UPDATE,
 * CHANGE, MODIFY, OR OTHERWISE SUPPORT THIS SOFTWARE.
 */

#ifndef CFS_SC_UTILITIES_H
#define CFS_SC_UTILITIES_H

// Thids file defines the function working on the two file
// used by our CFS wrapper.
// These file are the user cfs mtab database ande the user 
// cfs fstab file.
//////////////////////////////////////////////////////////

// Some specific constants relatives to the dbm database
#define DB_BLOC_SIZE           0
#define OPEN_CREATE_DB_RIGHTS  0744
#define NO_FATAL_FUNCT         0
#define MAX_DBNAME_SIZE        512

// In all the next functions the quiet flag indicates 
// that the function mustn't print any message.
/////////////////////////////////////////////////////

// These functions are used to manage the user cfs mtab database.
// In this database, are registred the clear name of the 
// currently attached directories (as key) and for futur usage 
// the ciphered name of this directories (as data).
/////////////////////////////////////////////////////////////////

// Function used to create the database
int mtab_create(int quiet);

// Function used to remove the database
int mtab_delete(int quiet);

// Function used to add a directory in the database
int add_dir_in_mtab(char *clearDirName, char *cipheredDirName, int quiet);

// Function used to delete a directory from the database
int delete_dir_in_mtab(char *clearDirName, int quiet);

// Function used to test if the given clear name is in the database
int is_dir_in_mtab(char *clearDirName, int *result, int quiet);

// Function used to get a clear name stored in the database
int give_dir_in_mtab(char *clearDirName, int *result, int quiet);

// Function used to test if the database is empty
int is_mtab_empty(int *result, int quiet);


// These functions are used to manage the user cfs fstab file.
// This file stores correspondences between ciphered directory
// names and clears names with a flag indicating if the directory
// can be auto attached.
// This file can be filled by the user or by the cmkdir_SC binary.
// This file is not required. Its format is:
//
// ciphered_directory_name <tab> clear_name <tab> auto_attach_flag
//
//  - ciphered_directory_name must be an absolut path.
//  - auto_attach_flag is auto or noauto.
//////////////////////////////////////////////////////////////

// Function used to create and initialize the file
int create_cfs_fstab(int quiet);

// Function used to test if the file exist
int is_cfs_fstab_exist(int *result, int quiet);

// Function used to add a new entry on the file
int add_cfs_fstab_entry(char *cipheredDirName, int quiet);

// Function used to get a specific entry of the file
int give_a_cfs_fstab_entry(char *cipheredDirName, char *clearDirName,
	int lineNR, int *found, int quiet);

// Function used to get the clear name corresponding to the given ciphered name
int get_a_cfs_fstab_clear_name(char *cipheredDirName, char *clearDirName,
	int *found, int quiet);

#endif /* CFS_SC_UTILITIES_H */
