/*
 *
 * This program is written by
 * Vincent Guerin
 * and Ludovic Rousseau <ludovic.rousseau@gemplus.com>
 *
 * $Id: pam_smartcard_utils.h,v 1.7 2002/03/11 15:03:13 rousseau Exp $
 *
 */

/*
 * Copyright  2000 Gemplus
 * Av. du Pic de Bertagne - Parc d'Activits de Gmenos
 * BP 100 - 13881 Gmenos CEDEX
 * 
 * Everyone is allowed to redistribute and use this source  (source
 * code)  and binary (object code),  with or  without modification,
 * under some conditions:
 * 
 *  - Everyone  must  retain  and/or  reproduce the above copyright
 *    notice,  and the below  disclaimer of warranty and limitation
 *    of liability  for redistribution and use of these source code
 *    and object code.
 * 
 *  - Everyone  must  ask a  specific prior written permission from
 *    Gemplus to use the name of Gemplus.
 * 
 * DISCLAIMER OF WARRANTY
 * 
 * THIS CODE IS PROVIDED "AS IS",  WITHOUT ANY WARRANTY OF ANY KIND
 * (INCLUDING,  BUT  NOT  LIMITED  TO,  THE IMPLIED  WARRANTIES  OF
 * MERCHANTABILITY  AND FITNESS FOR  A  PARTICULAR PURPOSE)  EITHER
 * EXPRESS OR IMPLIED.  GEMPLUS DOES NOT WARRANT THAT THE FUNCTIONS
 * CONTAINED  IN THIS SOFTWARE WILL MEET THE USER'S REQUIREMENTS OR
 * THAT THE OPERATION OF IT WILL BE UNINTERRUPTED OR ERROR-FREE. NO
 * USE  OF  ANY  CODE  IS  AUTHORIZED  HEREUNDER EXCEPT UNDER  THIS
 * DISCLAIMER.
 * 
 * LIMITATION OF LIABILITY
 * 
 * GEMPLUS SHALL NOT BE LIABLE FOR INFRINGEMENTS OF  THIRD  PARTIES
 * RIGHTS. IN NO EVENTS, UNLESS REQUIRED BY APPLICABLE  LAW,  SHALL
 * GEMPLUS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES OF ANY CHARACTER  INCLUDING,
 * WITHOUT LIMITATION, DAMAGES FOR LOSS OF GOODWILL, WORK STOPPAGE,
 * COMPUTER FAILURE OR MALFUNCTION, OR ANY AND ALL OTHER DAMAGES OR
 * LOSSES, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. ALSO,
 * GEMPLUS IS  UNDER NO  OBLIGATION TO MAINTAIN,  CORRECT,  UPDATE,
 * CHANGE, MODIFY, OR OTHERWISE SUPPORT THIS SOFTWARE.
 */

#ifndef PAM_SC_UTILS_H
#define PAM_SC_UTILS_H

#include <gmp.h>
#include <winscard.h>

#include <security/pam_modules.h>

// This file defines the functions and constants used by the
// PAM smartcard authentication module.
// it also defines the pam_log function also used by the PAM CFS for
/// smartcard module to log some events in the computer loig files.
//////////////////////////////////////////////////////////////////// 

// The log identifier constants
#define AUTOLOCK_IDENT        1
#define PAM_SMARTCARD_AUTH    2
#define PAM_SMARTCARD_SESSION 3
#define PAM_SMARTCARD         4
#define PAM_CFS_SC            5

// Argument parsing    
///////////////////

// The administrator want to use the local database
// to get the public key. Default is *NOT* NIS
#define     PAM_NIS              0

// Actually ignored... 
#define     PAM_VERBOSE_ARG      2
#define     PAM_NWARN_ARG        4
#define     PAM_UFIRST_ARG       8
#define     PAM_TFIRST_ARG      16
#define     PAM_MAPPED_ARG      64

// Function prototypes 
//////////////////////

// This function gets and verifies the public key of the user from NIS
int nis_get_user_info(char *username, mpz_t *pubExpo, mpz_t *pubModulo);

// This function gets and verifies  the public key 
// of the user from a dbm database.
int localdb_get_user_info(char *username, char *localDbName,
	mpz_t *pubExpo, mpz_t *pubModulo);

// This function parses the optionnal argument given in 
// the application configuration file.
int pam_parse(int argc, const char **argv, int *ctrl, char *configfile);

// This fonction log some events in the appropriate system log file
void pam_log(int processIdent, int err, const char *format, ...);

// This function is used by the module to perform its communication
int pam_conversate(pam_handle_t * pamh, char *pin);

// Random number generator for the authentification part of the module
int generaterandom(unsigned char *pucBuffer, unsigned long dwLength);

// Return EXIT_SUCCESS if the user is on the autolock exclude user file.
// EXIT_FAILURE else.
int is_user_excluded(char *username);

// This function verifies the RSA signature of data.
int verify_signature(mpz_t *pubExpo, mpz_t *pubModulo, unsigned char *signature,
	unsigned char *initialData, int initialDataSize);

// Functions used by the session part to manage the session pid list
// The first entry of this list is the pid of autolock process,
// This entry is permanent.
// The next pid are the pid of the process which have opened
// a session.
/////////////////////////////////////////////////////////////////////

// Writes the autolock pid and initialize the session process pid list
int listInit(char *userName);

// Returns the pid of the autolock daemon
int getAutolockPid(int *pid);

// Writes the autolock pid at the beginning of the autolock pid file
int setAutolockPid(int autolockPid);

// Return the pid of the nr opened session if it exists
int getSessionPid(int *pid, int *result, int nr);

// Puts a session process pid in the list
int putPidInList(int pid);

// Deletes a session process pid of the list
int deletePidInList(int pid);

// Sets result to 1 if the pid is on the list, set to 0 else
int pidInList(int pid, int *result);

// Sets result to 1 if the list is empty, set to 0 else
int listEmpty(int *result);

// This function check the lsit in order to verify that all the 
// registred sessions are still alive
int cleanupList();


// Functions use to warn the user of differents events
//////////////////////////////////////////////////////

// A little melody
void bell();

// Just one beep
void beep();

#endif /* PAM_SC_UTILS_H */
