/*
 *
 * This program is written by
 * Vincent Guerin
 * and Ludovic Rousseau <ludovic.rousseau@gemplus.com>
 *
 * $Id: publickey_db_functions.h,v 1.3 2002/03/07 10:33:05 rousseau Exp $
 *
 */

/*
 * Copyright  2000 Gemplus
 * Av. du Pic de Bertagne - Parc d'Activits de Gmenos
 * BP 100 - 13881 Gmenos CEDEX
 * 
 * Everyone is allowed to redistribute and use this source  (source
 * code)  and binary (object code),  with or  without modification,
 * under some conditions:
 * 
 *  - Everyone  must  retain  and/or  reproduce the above copyright
 *    notice,  and the below  disclaimer of warranty and limitation
 *    of liability  for redistribution and use of these source code
 *    and object code.
 * 
 *  - Everyone  must  ask a  specific prior written permission from
 *    Gemplus to use the name of Gemplus.
 * 
 * DISCLAIMER OF WARRANTY
 * 
 * THIS CODE IS PROVIDED "AS IS",  WITHOUT ANY WARRANTY OF ANY KIND
 * (INCLUDING,  BUT  NOT  LIMITED  TO,  THE IMPLIED  WARRANTIES  OF
 * MERCHANTABILITY  AND FITNESS FOR  A  PARTICULAR PURPOSE)  EITHER
 * EXPRESS OR IMPLIED.  GEMPLUS DOES NOT WARRANT THAT THE FUNCTIONS
 * CONTAINED  IN THIS SOFTWARE WILL MEET THE USER'S REQUIREMENTS OR
 * THAT THE OPERATION OF IT WILL BE UNINTERRUPTED OR ERROR-FREE. NO
 * USE  OF  ANY  CODE  IS  AUTHORIZED  HEREUNDER EXCEPT UNDER  THIS
 * DISCLAIMER.
 * 
 * LIMITATION OF LIABILITY
 * 
 * GEMPLUS SHALL NOT BE LIABLE FOR INFRINGEMENTS OF  THIRD  PARTIES
 * RIGHTS. IN NO EVENTS, UNLESS REQUIRED BY APPLICABLE  LAW,  SHALL
 * GEMPLUS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES OF ANY CHARACTER  INCLUDING,
 * WITHOUT LIMITATION, DAMAGES FOR LOSS OF GOODWILL, WORK STOPPAGE,
 * COMPUTER FAILURE OR MALFUNCTION, OR ANY AND ALL OTHER DAMAGES OR
 * LOSSES, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. ALSO,
 * GEMPLUS IS  UNDER NO  OBLIGATION TO MAINTAIN,  CORRECT,  UPDATE,
 * CHANGE, MODIFY, OR OTHERWISE SUPPORT THIS SOFTWARE.
 */

#ifndef PUBLICKEY_BD_FUNCTIONS_H
#define PUBLICKEY_BD_FUNCTIONS_H

// This file defines the functions used to manage the public key 
// database. This database stores the names of the registred users 
// and their cerrtified corresponding public key.
//////////////////////////////////////////////////////////////////

// Some specific constants
#define DB_BLOC_SIZE           0
#define OPEN_CREATE_DB_RIGHTS  0744
#define NO_FATAL_FUNCT         0

#define SST_LOCK_FILE_RIGHTS   0700

// Error return codes
#define ERR_USR_ALREADY_IN_DB  -4
#define USR_NOT_IN_DB          -5

// Function used to test if the queried database exists
int test_db(char *dbFileName);

// Function used to manage a lock file, in order to ensure the unicity
// of the process operating on the public key database
int test_lock();

// Creates a database
int public_key_db_create_db(char *publicKeyDest, int new);

// Searchs if the user is registred in the database
int publickey_db_user_in_db(char *username, char *publicKeyDest);

// Creates a new entry for a user in the database
int publickey_db_add_user(char *username, unsigned char *publicKey,
	unsigned char *signature, char *publicKeyDest, int forcedFlag);

// Deletes a user in the database
int publickey_db_delete_user(char *username, char *publicKeyDest);

// Lists users registred in the database
int publickey_db_list_user(char *publicKeyDest, char ***list, int *size);

// This function search in the database if all the public keys
// are differents. In other case the meaning is probably that a
// card was reused and it's previous user wasn't removed of the database
int publickey_db_verify_key(char *publicKeyDest, char *user_1, char *user_2);

#endif /* PUBLICKEY_BD_FUNCTIONS_H */
