/*
 *
 * This program is written by
 * Vincent Guerin
 * and Ludovic Rousseau <ludovic.rousseau@gemplus.com>
 *
 * $Id: smartcard_functions.h,v 1.6 2002/03/11 15:04:56 rousseau Exp $
 *
 */

/*
 * Copyright  2000 Gemplus
 * Av. du Pic de Bertagne - Parc d'Activits de Gmenos
 * BP 100 - 13881 Gmenos CEDEX
 * 
 * Everyone is allowed to redistribute and use this source  (source
 * code)  and binary (object code),  with or  without modification,
 * under some conditions:
 * 
 *  - Everyone  must  retain  and/or  reproduce the above copyright
 *    notice,  and the below  disclaimer of warranty and limitation
 *    of liability  for redistribution and use of these source code
 *    and object code.
 * 
 *  - Everyone  must  ask a  specific prior written permission from
 *    Gemplus to use the name of Gemplus.
 * 
 * DISCLAIMER OF WARRANTY
 * 
 * THIS CODE IS PROVIDED "AS IS",  WITHOUT ANY WARRANTY OF ANY KIND
 * (INCLUDING,  BUT  NOT  LIMITED  TO,  THE IMPLIED  WARRANTIES  OF
 * MERCHANTABILITY  AND FITNESS FOR  A  PARTICULAR PURPOSE)  EITHER
 * EXPRESS OR IMPLIED.  GEMPLUS DOES NOT WARRANT THAT THE FUNCTIONS
 * CONTAINED  IN THIS SOFTWARE WILL MEET THE USER'S REQUIREMENTS OR
 * THAT THE OPERATION OF IT WILL BE UNINTERRUPTED OR ERROR-FREE. NO
 * USE  OF  ANY  CODE  IS  AUTHORIZED  HEREUNDER EXCEPT UNDER  THIS
 * DISCLAIMER.
 * 
 * LIMITATION OF LIABILITY
 * 
 * GEMPLUS SHALL NOT BE LIABLE FOR INFRINGEMENTS OF  THIRD  PARTIES
 * RIGHTS. IN NO EVENTS, UNLESS REQUIRED BY APPLICABLE  LAW,  SHALL
 * GEMPLUS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES OF ANY CHARACTER  INCLUDING,
 * WITHOUT LIMITATION, DAMAGES FOR LOSS OF GOODWILL, WORK STOPPAGE,
 * COMPUTER FAILURE OR MALFUNCTION, OR ANY AND ALL OTHER DAMAGES OR
 * LOSSES, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. ALSO,
 * GEMPLUS IS  UNDER NO  OBLIGATION TO MAINTAIN,  CORRECT,  UPDATE,
 * CHANGE, MODIFY, OR OTHERWISE SUPPORT THIS SOFTWARE.
 */

#ifndef SMARTCARD_FUNCTIONS_H
#define SMARTCARD_FUNCTIONS_H

#include <winscard.h>

SCARDHANDLE card;
SCARDCONTEXT context;

// This file defines the functions working on the card.
///////////////////////////////////////////////////////

// Return error codes
#define ERROR_CARD_ALREADY_FORMATED -2
#define ERROR_CARD_NOT_FORMATED     -3
#define ERROR_VERIFY_PIN            -7

#define ISO7816_ERROR_TEXT_SIZE    200

// The purpose of this function is to format the card
// for the smartsession applications.
// In brief : Create the directory, the files, load the secret 
// codes and generate the RSA key and the CFS secret.
int formatcard(char *passphrase, int passphraseLen);

// This function is usefull to change the user PIN or to unlock the card
// It's not required to know the current user PIN, by it required the
// passphrase and so it's reserved to the administrator..
int pin_mngmt(char *passphrase, int passphraseLen, char *new_pin);

// This function allows to assign a card to an user.
//  - The username is stored in the card.
//  - The public key stored on the card is returned (to be signed 
//    and store in the public keys database...).
int perso_card(char *passphrase, int passphraseLen, char *username,
	char *publicKeyDest, unsigned char *publicKey);

// This function is used change the user pin. It required the current 
// user PIN, so it can be used by the user himself.
int change_pin(char *current_pin, char *new_pin);

// This function is used to create the certification card of
// the administrator.
int generate_certification_part(char *passphrase, int passphraseLen);

// This function is used to install the certification part of the 
// public key in a computer
int install_certification_part(char *passphrase, int passphraseLen);

// Function used to sign the user public key using the root certification key
int sign_user_public_key(unsigned char *publicKey, char *userName,
	char *passphrase, unsigned char *signature, int twoReaders);

// Read the CFS secret stored in the card
int get_cfs_secret(char *userPin, unsigned char *cfsSecret, int quiet);

// Uncipher the cipher CFS secret to perfom the restore secret operation
int restore_cfs_secret(char *passphrase, int passphraseLen,
	unsigned char *cipheredCfsSecret, unsigned char *cfsSecret);

/*
* Convert return codes from APDU EXCHANGE into strig description
*
* @param SW1 @param SW2
*
* Return a pointer to description character string
*/
char *ISO7816_ErrorStr(unsigned char sw1, unsigned char sw2);

/**
* Establish a new context to the card reader.
*
* @param reader Which number of cardreader should be used
*
* @return 0 on success, -1 if failed
*
*/
int pcsc_init(int reader);

/**
* Release the card context
*
*/
void pcsc_release();

/**
* Erase the Card Content
*
*/
int erasecard();

/**
* Get Card Directory
*
*/
int dircard();

#endif /* SMARTCARD_FUNCTIONS_H */
