/*
 *
 * This program is written by
 * Vincent Guerin
 * and Ludovic Rousseau <ludovic.rousseau@gemplus.com>
 *
 * Code changed to use PCSC instead of TLP calls
 * November 2001 - Domenico Taddei <domenico.taddei@proteam-group.it>
 *
 * $Id: pam_smartcard.c,v 1.10 2002/03/12 10:32:41 rousseau Exp $
 *
 */

/*
 * Copyright  2000 Gemplus
 * Av. du Pic de Bertagne - Parc d'Activits de Gmenos
 * BP 100 - 13881 Gmenos CEDEX
 * 
 * Everyone is allowed to redistribute and use this source  (source
 * code)  and binary (object code),  with or  without modification,
 * under some conditions:
 * 
 *  - Everyone  must  retain  and/or  reproduce the above copyright
 *    notice,  and the below  disclaimer of warranty and limitation
 *    of liability  for redistribution and use of these source code
 *    and object code.
 * 
 *  - Everyone  must  ask a  specific prior written permission from
 *    Gemplus to use the name of Gemplus.
 * 
 * DISCLAIMER OF WARRANTY
 * 
 * THIS CODE IS PROVIDED "AS IS",  WITHOUT ANY WARRANTY OF ANY KIND
 * (INCLUDING,  BUT  NOT  LIMITED  TO,  THE IMPLIED  WARRANTIES  OF
 * MERCHANTABILITY  AND FITNESS FOR  A  PARTICULAR PURPOSE)  EITHER
 * EXPRESS OR IMPLIED.  GEMPLUS DOES NOT WARRANT THAT THE FUNCTIONS
 * CONTAINED  IN THIS SOFTWARE WILL MEET THE USER'S REQUIREMENTS OR
 * THAT THE OPERATION OF IT WILL BE UNINTERRUPTED OR ERROR-FREE. NO
 * USE  OF  ANY  CODE  IS  AUTHORIZED  HEREUNDER EXCEPT UNDER  THIS
 * DISCLAIMER.
 * 
 * LIMITATION OF LIABILITY
 * 
 * GEMPLUS SHALL NOT BE LIABLE FOR INFRINGEMENTS OF  THIRD  PARTIES
 * RIGHTS. IN NO EVENTS, UNLESS REQUIRED BY APPLICABLE  LAW,  SHALL
 * GEMPLUS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES OF ANY CHARACTER  INCLUDING,
 * WITHOUT LIMITATION, DAMAGES FOR LOSS OF GOODWILL, WORK STOPPAGE,
 * COMPUTER FAILURE OR MALFUNCTION, OR ANY AND ALL OTHER DAMAGES OR
 * LOSSES, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. ALSO,
 * GEMPLUS IS  UNDER NO  OBLIGATION TO MAINTAIN,  CORRECT,  UPDATE,
 * CHANGE, MODIFY, OR OTHERWISE SUPPORT THIS SOFTWARE.
 */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <errno.h>
#include <signal.h>
#include <pwd.h>
#include <winscard.h>
#include <gmp.h>

#include <security/pam_modules.h>

#include "config.h"
#include "global.h"
#include "apdu.h"
#include "crypto_functions.h"
#include "pam_smartcard_utils.h"
#include "smartcard_functions.h"

// PAM services offered
#define PAM_SM_AUTH
#define PAM_SM_SESSION

// Cleaning up function for the data mechanism
void cleanup_func(pam_handle_t * pamh, void *data, int error_status)
{
	// Void
}

PAM_EXTERN int pam_sm_authenticate(pam_handle_t * pamh, int flags,
	int argc, const char **argv)
{
	int i, ctrl, rCode;
	int ret;
	mpz_t pubExpo, pubModulo;
	BYTE recv[BUF_SIZE];
	BYTE apdu[BUF_SIZE];
	DWORD recvlen;
	SCARD_IO_REQUEST pio_recv_pci;
	char *userName;
	char userNameInCard[USER_NAME_MAX_SIZE];
	char localDbName[MAX_LEN_OF_DB_NAME];
	char *pin;

	struct MD5Context md5Context;

	unsigned char refNumber[SIZEOF_REF_NUMBER];
	unsigned char dataSigned[HASH_PADDING_SIZE];
	unsigned char dataSignedInvert[HASH_PADDING_SIZE];
	unsigned char digest[DIGEST_SIZE];

	// First get the arguments
	rCode = pam_parse(argc, argv, &ctrl, localDbName);

	if (rCode == EXIT_FAILURE)
	{
		pam_log(PAM_SMARTCARD_AUTH, LOG_ERR,
			"UNABLE to parse the arguments of the module");
		return (PAM_USER_UNKNOWN);
	}

	// Next get the user name
	rCode = pam_get_user(pamh, (const char **) &userName, NULL);

	if ((rCode != PAM_SUCCESS) || (!userName))
	{
		pam_log(PAM_SMARTCARD_AUTH, LOG_ERR, "Unable to get the user name");
		return (PAM_USER_UNKNOWN);
	}

	// Now tell to the user he have to insert his card
	// Wait for it...and for the user PIN...
	//////////////////////////////////////////////////

	pin = (char *) malloc(USR_PIN_SIZE * sizeof(char));

	rCode = pam_conversate(pamh, pin);

	if (rCode != PAM_SUCCESS)
	{
		pam_log(PAM_SMARTCARD_AUTH, LOG_CRIT, "Conversate failed");
		return (PAM_AUTHINFO_UNAVAIL);
	}

	// Put the PIN code in the pamh handle structure
	// to provide it to hypothetical other modules
	pam_set_data(pamh, PAM_PIN_DATA_NAME, pin, cleanup_func);

	// Connection to the card
	rCode = pcsc_init(0);

	if (rCode != SCARD_S_SUCCESS)
	{
		pam_log(PAM_SMARTCARD_AUTH, LOG_INFO,
			"Unable to be connected to the card");
		pcsc_release();
		return (PAM_AUTH_ERR);
	}

	// Select the PAM directory on the card
	///////////////////////////////////////

	apdu[0] = ISO_COMPATIBLE;
	apdu[1] = SELECT_FILE;
	apdu[2] = P1_SELECT_DF_WHEN_MF_SELECTED;
	apdu[3] = P2_NO_REPONSE_REQUIRED;
	apdu[4] = SFI_SIZE;

	apdu[5] = PAM_DF_SFI;
	apdu[6] = SFI_NULL;

	recvlen = 2;
	rCode = SCardTransmit(card, SCARD_PCI_T0, apdu, 5 + SFI_SIZE,
		&pio_recv_pci, recv, &recvlen);

	if (rCode != SCARD_S_SUCCESS)
	{
		pam_log(PAM_SMARTCARD_AUTH, LOG_INFO,
			"Error sending APDU select PAM DF: %s",
			pcsc_stringify_error(rCode));
		pcsc_release();
		return (PAM_AUTH_ERR);
	}

	if (!Successfull_Command(recv[recvlen - 2], recv[recvlen - 1]))
	{
		pam_log(PAM_SMARTCARD_AUTH, LOG_INFO, "Error Select PAM DF: %s",
			ISO7816_ErrorStr(recv[recvlen - 2], recv[recvlen - 1]));
		pcsc_release();
		return (PAM_AUTH_ERR);
	}

	// Get the user name stored in the card
	///////////////////////////////////////

	apdu[0] = ISO_COMPATIBLE;
	apdu[1] = READ_BINARY;
	apdu[2] = P1_SELECT_READ_FROM_EF | USER_NAME_SFI;
	apdu[3] = P2_OFFSET_NULL;
	apdu[4] = USER_NAME_MAX_SIZE;
	recvlen = USER_NAME_MAX_SIZE + 2;
	rCode = SCardTransmit(card, SCARD_PCI_T0, apdu, 5, &pio_recv_pci, recv,
		&recvlen);

	if (rCode != SCARD_S_SUCCESS)
	{
		pam_log(PAM_SMARTCARD_AUTH, LOG_INFO,
			"Error sending APDU read user name: %s",
			pcsc_stringify_error(rCode));
		pcsc_release();
		return (PAM_AUTH_ERR);
	}

	if (!Successfull_Command(recv[recvlen - 2], recv[recvlen - 1]))
	{
		pcsc_release();
		pam_log(PAM_SMARTCARD_AUTH, LOG_INFO,
			"Error Read user name in the card: %s",
			ISO7816_ErrorStr(recv[recvlen - 2], recv[recvlen - 1]));
		return (PAM_AUTH_ERR);
	}

	memcpy((unsigned char *) userNameInCard, recv, USER_NAME_MAX_SIZE);

	// Compare the username name known by PAM and the user 
	// name stored in the card
	//////////////////////////////////////////////////////

	// userName may be "" if nothing is entered
	if (strcmp(userNameInCard, userName) != 0)
	{
		pam_log(PAM_SMARTCARD_AUTH, LOG_INFO,
			"Card user name (%s) does not match login user name (%s)",
			userNameInCard, userName);

		pcsc_release();
		return (PAM_AUTH_ERR);
	}

	// Verify the user PIN code
	///////////////////////////

	apdu[0] = ISO_COMPATIBLE;
	apdu[1] = VERIFY;
	apdu[2] = P1_VERIFY;
	apdu[3] = USR_PIN_CODE_NUMBER;
	apdu[4] = SECRET_CODE_SIZE;

	for (i = 0; i < USR_PIN_SIZE; i++)
		apdu[5 + i] = pin[i] - '0';

	// Padd it with 0

	for (i = USR_PIN_SIZE; i < SECRET_CODE_SIZE; i++)
		apdu[5 + i] = '0';

	recvlen = 2;
	rCode = SCardTransmit(card, SCARD_PCI_T0, apdu, 5 + SECRET_CODE_SIZE,
		&pio_recv_pci, recv, &recvlen);

	if (rCode != SCARD_S_SUCCESS)
	{
		pam_log(PAM_SMARTCARD_AUTH, LOG_INFO,
			"Error sending APDU verify user PIN: %s",
			pcsc_stringify_error(rCode));
		pcsc_release();
		return (PAM_AUTH_ERR);
	}

	if (!Successfull_Command(recv[recvlen - 2], recv[recvlen - 1]))
	{
		pam_log(PAM_SMARTCARD_AUTH, LOG_INFO, "Error Verify user PIN: %s",
			ISO7816_ErrorStr(recv[recvlen - 2], recv[recvlen - 1]));
		pcsc_release();
		return (PAM_AUTH_ERR);
	}

	// Select the private key file 
	//////////////////////////////

	apdu[0] = ISO_COMPATIBLE;
	apdu[1] = SELECT_FILE;
	apdu[2] = P1_SELECT_EF_UNDER_CURRENT_DF;
	apdu[3] = P2_NO_REPONSE_REQUIRED;
	apdu[4] = SFI_SIZE;
	apdu[5] = PAM_DF_SFI;
	apdu[6] = RSA_KEY_SFI;

	recvlen = 2;
	rCode = SCardTransmit(card, SCARD_PCI_T0, apdu, 5 + SFI_SIZE,
		&pio_recv_pci, recv, &recvlen);

	if (rCode != SCARD_S_SUCCESS)
	{
		pam_log(PAM_SMARTCARD_AUTH, LOG_INFO,
			"Error Sending APDU select private key file: %s",
			pcsc_stringify_error(rCode));
		pcsc_release();
		return (PAM_AUTH_ERR);
	}

	if (!Successfull_Command(recv[recvlen - 2], recv[recvlen - 1]))
	{
		pam_log(PAM_SMARTCARD_AUTH, LOG_INFO,
			"Error Select private key file: %s",
			ISO7816_ErrorStr(recv[recvlen - 2], recv[recvlen - 1]));
		pcsc_release();
		return (PAM_AUTH_ERR);
	}

	// Select Crypto Context : Sign MD5 / RSA 512
	/////////////////////////////////////////////

	apdu[0] = PROPRIETARY;
	apdu[1] = SELECT_CRYPTO_CONTEXT;
	apdu[2] = RSA_KEY_SFI;
	apdu[3] = P2_MODE_RSA_MD5_SIGN;
	apdu[4] = NULL_BYTE;

	recvlen = 2;
	rCode = SCardTransmit(card, SCARD_PCI_T0, apdu, 5, &pio_recv_pci, recv,
		&recvlen);

	if (rCode != SCARD_S_SUCCESS)
	{
		pam_log(PAM_SMARTCARD_AUTH, LOG_INFO,
			"Error sending APDU select crypto context: %s",
			pcsc_stringify_error(rCode));
		pcsc_release();
		return (PAM_AUTH_ERR);
	}

	if (!Successfull_Command(recv[recvlen - 2], recv[recvlen - 1]))
	{
		pam_log(PAM_SMARTCARD_AUTH, LOG_INFO,
			"Error Select crypto context: %s",
			ISO7816_ErrorStr(recv[recvlen - 2], recv[recvlen - 1]));
		pcsc_release();
		return (PAM_AUTH_ERR);
	}

	// Now, we have to test if the keys are good RSA keys pair
	//////////////////////////////////////////////////////////

	// Generate the random number
	rCode = generaterandom(refNumber, SIZEOF_REF_NUMBER);

	if (rCode != EXIT_SUCCESS)
	{
		pam_log(PAM_SMARTCARD_AUTH, LOG_ERR,
			"Unable to generate the random number");
		pcsc_release();
		return (PAM_AUTH_ERR);
	}

	// Hash the random number
	MD5Init(&md5Context);
	MD5Update(&md5Context, refNumber, SIZEOF_REF_NUMBER);
	MD5Final(digest, &md5Context);

	apdu[0] = PROPRIETARY;
	apdu[1] = INIT_HASHED_DATA;
	apdu[2] = NULL_BYTE;
	apdu[3] = NULL_BYTE;
	apdu[4] = DIGEST_SIZE;

	// Invert the digest
	for (i = 0; i < DIGEST_SIZE; i++)
		apdu[5 + i] = digest[DIGEST_SIZE - i - 1];

	// Send the digest to the card 
	recvlen = 2;
	rCode = SCardTransmit(card, SCARD_PCI_T0, apdu, 5 + DIGEST_SIZE,
		&pio_recv_pci, recv, &recvlen);

	if (rCode != SCARD_S_SUCCESS)
	{
		pam_log(PAM_SMARTCARD_AUTH, LOG_INFO,
			"Error Sending APDU Init Hashed Data: %s",
			pcsc_stringify_error(rCode));
		pcsc_release();
		return (PAM_AUTH_ERR);
	}

	if (!Successfull_Command(recv[recvlen - 2], recv[recvlen - 1]))
	{
		pam_log(PAM_SMARTCARD_AUTH, LOG_INFO, "Error Init Hashed Data: %s",
			ISO7816_ErrorStr(recv[recvlen - 2], recv[recvlen - 1]));
		pcsc_release();
		return (PAM_AUTH_ERR);
	}

	// Tell the card to sign the digest
	apdu[0] = PROPRIETARY;
	apdu[1] = PK_SIGN;
	apdu[2] = NULL_BYTE;
	apdu[3] = NULL_BYTE;
	apdu[4] = HASH_PADDING_SIZE;

	recvlen = HASH_PADDING_SIZE + 2;
	rCode = SCardTransmit(card, SCARD_PCI_T0, apdu, 5, &pio_recv_pci, recv,
		&recvlen);
	if (rCode != SCARD_S_SUCCESS)
	{
		pam_log(PAM_SMARTCARD_AUTH, LOG_INFO,
			"Error Sending APDU sign the data: %s",
			pcsc_stringify_error(rCode));
		pcsc_release();
		return (PAM_AUTH_ERR);
	}

	if (!Successfull_Command(recv[recvlen - 2], recv[recvlen - 1]))
	{
		pam_log(PAM_SMARTCARD_AUTH, LOG_INFO, "Error Sign the data: %s",
			ISO7816_ErrorStr(recv[recvlen - 2], recv[recvlen - 1]));
		pcsc_release();
		return (PAM_AUTH_ERR);
	}

	memcpy(dataSigned, recv, HASH_PADDING_SIZE);

	// Card disconnection
	pcsc_release();

	// Invert the signature sent by the card
	for (i = 0; i < HASH_PADDING_SIZE; i++)
		dataSignedInvert[HASH_PADDING_SIZE - 1 - i] = dataSigned[i];

	// Get the user public key
	// In a local database or by NIS
	////////////////////////////////
	if (ctrl & PAM_NIS)
		rCode = nis_get_user_info(userName, &pubExpo, &pubModulo);
	else
		rCode = localdb_get_user_info(userName, localDbName, &pubExpo,
			&pubModulo);

	if (rCode != PAM_SUCCESS)
	{
		pam_log(PAM_SMARTCARD_AUTH, LOG_INFO,
			"Unable to get user public key");

		mpz_clear(pubExpo);
		mpz_clear(pubModulo);

		return (PAM_AUTH_ERR);
	}

	// Verify if the result returned by the card is the right 
	// signature of the reference number
	if (verify_signature(&pubExpo, &pubModulo, dataSignedInvert,
			refNumber, SIZEOF_REF_NUMBER) == EXIT_SUCCESS)
	{
		// Ok, it's the good private key
		pam_log(PAM_SMARTCARD_AUTH, LOG_INFO, "User %s authenticated",
			userName);
		ret = PAM_SUCCESS;
	}
	else
	{
		// It's not the good private key
		pam_log(PAM_SMARTCARD_AUTH, LOG_INFO,
			"Bad private key: user %s NOT authenticated", userName);
		ret = PAM_AUTH_ERR;
	}

	mpz_clear(pubExpo);
	mpz_clear(pubModulo);

	return ret;
}

PAM_EXTERN int pam_sm_setcred(pam_handle_t * pamh, int flags,
	int argc, const char **argv)
{
	// Manage the user credentials

	// Nothing to do...
	return (PAM_SUCCESS);
}

PAM_EXTERN int pam_sm_open_session(pam_handle_t * pamh, int flags,
	int argc, const char **argv)
{
	int pid, rCode, i;
	char sessionPid[BUF_TMP_SIZE];
	const char *userName;
	unsigned char csn[CSN_SIZE];
	BYTE recv[BUF_SIZE];
	BYTE apdu[BUF_SIZE];
	DWORD recvlen;
	SCARD_IO_REQUEST pio_recv_pci;
	char csnTxt[CSN_SIZE * 2 + 1];
	char userNameInCard[USER_NAME_MAX_SIZE];

	// First get the user name
	rCode = pam_get_user(pamh, &userName, NULL);

	if (rCode != PAM_SUCCESS)
	{
		pam_log(PAM_SMARTCARD_SESSION, LOG_ERR,
			"Unable to get the user name");
		return (PAM_SESSION_ERR);
	}

	// Look for this user in the autolock excluded user file
	rCode = is_user_excluded((char *) userName);

	if (rCode == EXIT_SUCCESS)
	{
		// Nothing to do, this user is not concerned 
		// by the session part.
		pam_log(PAM_SMARTCARD_SESSION, LOG_INFO,
			"User %s excluded from the open session part", userName);
		return (PAM_SUCCESS);
	}

	// Connection to the card
	/////////////////////////

	rCode = pcsc_init(0);

	if (rCode != SCARD_S_SUCCESS)
	{
		pcsc_release();
		pam_log(PAM_SMARTCARD_SESSION, LOG_INFO,
			"Unable to be connected to the card");
		return (PAM_SESSION_ERR);
	}

	// Get the card serial number
	/////////////////////////////

	apdu[0] = PROPRIETARY;
	apdu[1] = GET_INFO;
	apdu[2] = P1_CARD_SERIAL_NUMBER;
	apdu[3] = P2_CARD_SERIAL_NUMBER;
	apdu[4] = CSN_SIZE;

	recvlen = CSN_SIZE + 2;
	rCode = SCardTransmit(card, SCARD_PCI_T0, apdu, 5, &pio_recv_pci, recv,
		&recvlen);
	if (rCode != SCARD_S_SUCCESS)
	{
		pam_log(PAM_SMARTCARD_SESSION, LOG_INFO,
			"Error Sending APDU Get card serial number : %s",
			pcsc_stringify_error(rCode));
		pcsc_release();
		return (PAM_SESSION_ERR);
	}

	if (!Successfull_Command(recv[recvlen - 2], recv[recvlen - 1]))
	{
		pam_log(PAM_SMARTCARD_SESSION, LOG_INFO,
			"Error Get card serial number: %s",
			ISO7816_ErrorStr(recv[recvlen - 2], recv[recvlen - 1]));
		pcsc_release();
		return (PAM_SESSION_ERR);
	}

	memcpy(csn, recv, CSN_SIZE);

	// Select the PAM directory on the card
	///////////////////////////////////////

	apdu[0] = ISO_COMPATIBLE;
	apdu[1] = SELECT_FILE;
	apdu[2] = P1_SELECT_DF_WHEN_MF_SELECTED;
	apdu[3] = P2_NO_REPONSE_REQUIRED;
	apdu[4] = SFI_SIZE;

	apdu[5] = PAM_DF_SFI;
	apdu[6] = SFI_NULL;

	recvlen = 2;
	rCode = SCardTransmit(card, SCARD_PCI_T0, apdu, 5 + SFI_SIZE,
		&pio_recv_pci, recv, &recvlen);

	if (rCode != SCARD_S_SUCCESS)
	{
		pam_log(PAM_SMARTCARD_SESSION, LOG_INFO,
			"Error sending APDU select PAM DF: %s",
			pcsc_stringify_error(rCode));
		pcsc_release();
		return (PAM_SESSION_ERR);
	}

	if (!Successfull_Command(recv[recvlen - 2], recv[recvlen - 1]))
	{
		pam_log(PAM_SMARTCARD_SESSION, LOG_INFO, "Error Select PAM DF: %s",
			ISO7816_ErrorStr(recv[recvlen - 2], recv[recvlen - 1]));
		pcsc_release();
		return (PAM_SESSION_ERR);
	}

	// Get the user name stored in the card
	///////////////////////////////////////

	apdu[0] = ISO_COMPATIBLE;
	apdu[1] = READ_BINARY;
	apdu[2] = P1_SELECT_READ_FROM_EF | USER_NAME_SFI;
	apdu[3] = P2_OFFSET_NULL;
	apdu[4] = USER_NAME_MAX_SIZE;

	recvlen = USER_NAME_MAX_SIZE + 2;
	rCode = SCardTransmit(card, SCARD_PCI_T0, apdu, 5, &pio_recv_pci, recv,
		&recvlen);

	if (rCode != SCARD_S_SUCCESS)
	{
		pam_log(PAM_SMARTCARD_SESSION, LOG_INFO,
			"Error sending APDU read user name: %s",
			pcsc_stringify_error(rCode));
		pcsc_release();
		return (PAM_SESSION_ERR);
	}

	if (!Successfull_Command(recv[recvlen - 2], recv[recvlen - 1]))
	{
		pam_log(PAM_SMARTCARD_SESSION, LOG_INFO,
			"Error Read user name in the card: %s",
			ISO7816_ErrorStr(recv[recvlen - 2], recv[recvlen - 1]));
		pcsc_release();
		return (PAM_SESSION_ERR);
	}

	memcpy((unsigned char *) userNameInCard, recv, CSN_SIZE);

	// Compare the username name known by PAM and the user name 
	// stored in the card
	///////////////////////////////////////////////////////////

	if (strncmp(userNameInCard, userName, strlen(userName)) != 0)
	{
		pam_log(PAM_SMARTCARD_SESSION, LOG_INFO,
			"Card user name (%s) does not match login user name (%s): autolock daemon not started",
			userNameInCard, userName);

		pcsc_release();
		return (PAM_SESSION_ERR);
	}

	// Convert the card serial number in text in order to
	// send it to the autolock daemon.

	for (i = 0; i < CSN_SIZE; i++)
		sprintf(csnTxt + 2 * i, "%02x", csn[i]);
	csnTxt[CSN_SIZE * 2] = '\0';

	// Disconection
	pcsc_release();

	// We can call the autolock daemon with 
	// with the add session command.
	///////////////////////////////////////

	sprintf(sessionPid, "%d", getpid());

	if ((pid = fork()) == 0)
	{
		int console_fd = 0;
		char console_fd_txt[10];

		// We are into the child

		console_fd = open("/dev/console", O_WRONLY);
		sprintf(console_fd_txt, "%d", console_fd);

		// Execute the autolock binary
		rCode = execlp(AUTOLOCK_DIR AUTOLOCK, AUTOLOCK,
			ADD_SESSION_CMD, userName, csnTxt, sessionPid,
			console_fd_txt, NULL);

		if (rCode == -1)
		{
			pam_log(PAM_SMARTCARD_SESSION, LOG_ERR,
				"Unable to execute the binary %s/%s: %s",
				AUTOLOCK_DIR, AUTOLOCK, strerror(errno));
			return (PAM_SESSION_ERR);
		}
	}
	else if (pid == -1)
	{
		pam_log(PAM_SMARTCARD_SESSION, LOG_ERR, "Unable to fork");
		return (PAM_SESSION_ERR);
	}
	else
	{
		// We are into the child
		// Wait for the end of the child
		waitpid(pid, NULL, 0);
	}

	return (PAM_SUCCESS);
}

PAM_EXTERN int pam_sm_close_session(pam_handle_t * pamh, int flags,
	int argc, const char **argv)
{
	int pid, rCode;
	const char *userName;
	char sessionPid[BUF_TMP_SIZE];

	// First get the user name
	rCode = pam_get_user(pamh, &userName, NULL);

	if (rCode != PAM_SUCCESS)
	{
		pam_log(PAM_SMARTCARD_SESSION, LOG_ERR,
			"Unable to get the user name");
		return (PAM_SESSION_ERR);
	}

	// Look for this user in the autolock excluded user file
	rCode = is_user_excluded((char *) userName);

	if (rCode == EXIT_SUCCESS)
	{
		// Nothing to do, this user is not concerned 
		// by the session part.
		pam_log(PAM_SMARTCARD_SESSION, LOG_INFO,
			"User %s excluded from the open session part", userName);
		return (PAM_SUCCESS);
	}

	// We can call the autolock daemon with 
	// with the add session command.
	///////////////////////////////////////

	sprintf(sessionPid, "%d", getpid());

	pid = fork();

	if (pid == 0)
	{
		// We are into the child

		// Execute the autolock binary
		rCode = execlp(AUTOLOCK_DIR AUTOLOCK, AUTOLOCK,
			RM_SESSION_CMD, sessionPid, NULL);

		if (rCode == -1)
		{
			pam_log(PAM_SMARTCARD_SESSION, LOG_ERR,
				"Unable to execute the binary %s/%s: %s",
				AUTOLOCK_DIR, AUTOLOCK, strerror(errno));
			return (PAM_SESSION_ERR);
		}
	}
	else if (pid == -1)
	{
		pam_log(PAM_SMARTCARD_SESSION, LOG_ERR, "Unable to fork");
		return (PAM_SESSION_ERR);
	}
	else
	{
		// We are into the child
		// Wait for the end of the child
		waitpid(pid, NULL, 0);
	}

	return (PAM_SUCCESS);
}

#ifdef PAM_STATIC
// Static module data 
struct pam_module _pam_smartcard_modstruct = {
	"pam_smartcard",
	pam_sm_authenticate,
	pam_sm_setcred,
	NULL,
	pam_sm_open_session,
	pam_sm_close_session,
	NULL
};
#endif
