/*
 *
 * This program is written by
 * Vincent Guerin
 * and Ludovic Rousseau <ludovic.rousseau@gemplus.com>
 *
 * $Id: autolock_ctrl.c,v 1.2 2001/01/03 09:58:01 rousseau Exp $
 *
 */

/*
 * Copyright  2000 Gemplus
 * Av. du Pic de Bertagne - Parc d'Activits de Gmenos
 * BP 100 - 13881 Gmenos CEDEX
 * 
 * Everyone is allowed to redistribute and use this source  (source
 * code)  and binary (object code),  with or  without modification,
 * under some conditions:
 * 
 *  - Everyone  must  retain  and/or  reproduce the above copyright
 *    notice,  and the below  disclaimer of warranty and limitation
 *    of liability  for redistribution and use of these source code
 *    and object code.
 * 
 *  - Everyone  must  ask a  specific prior written permission from
 *    Gemplus to use the name of Gemplus.
 * 
 * DISCLAIMER OF WARRANTY
 * 
 * THIS CODE IS PROVIDED "AS IS",  WITHOUT ANY WARRANTY OF ANY KIND
 * (INCLUDING,  BUT  NOT  LIMITED  TO,  THE IMPLIED  WARRANTIES  OF
 * MERCHANTABILITY  AND FITNESS FOR  A  PARTICULAR PURPOSE)  EITHER
 * EXPRESS OR IMPLIED.  GEMPLUS DOES NOT WARRANT THAT THE FUNCTIONS
 * CONTAINED  IN THIS SOFTWARE WILL MEET THE USER'S REQUIREMENTS OR
 * THAT THE OPERATION OF IT WILL BE UNINTERRUPTED OR ERROR-FREE. NO
 * USE  OF  ANY  CODE  IS  AUTHORIZED  HEREUNDER EXCEPT UNDER  THIS
 * DISCLAIMER.
 * 
 * LIMITATION OF LIABILITY
 * 
 * GEMPLUS SHALL NOT BE LIABLE FOR INFRINGEMENTS OF  THIRD  PARTIES
 * RIGHTS. IN NO EVENTS, UNLESS REQUIRED BY APPLICABLE  LAW,  SHALL
 * GEMPLUS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES OF ANY CHARACTER  INCLUDING,
 * WITHOUT LIMITATION, DAMAGES FOR LOSS OF GOODWILL, WORK STOPPAGE,
 * COMPUTER FAILURE OR MALFUNCTION, OR ANY AND ALL OTHER DAMAGES OR
 * LOSSES, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. ALSO,
 * GEMPLUS IS  UNDER NO  OBLIGATION TO MAINTAIN,  CORRECT,  UPDATE,
 * CHANGE, MODIFY, OR OTHERWISE SUPPORT THIS SOFTWARE.
 */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <signal.h>
#include <errno.h>
#include <getopt.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include "config.h"
#include "global.h"
#include "pam_smartcard_utils.h"

// This program is used to control the behaviour of the
// autolock daemon.
//
// To suspend the autolock daemon, we send it the SIGUSR1 signal
// To resume the autolock daemon, we send it the SIGUSR2 signal
//
// The pid of the daemon is readed in the autolock pid file.
////////////////////////////////////////////////////////////////

#define VERSION "1.0"

// Internal functions
/////////////////////

// Print the program version
void version(char *progName);

// Print the program usage informations
void help(char *progName);

int main(int argc, char **argv)
{
	FILE *fd;
	int nbReaded;
	int rCode, pid;
	int signalToSend = 0;
	struct stat statbuf;
	char progName[BUF_TMP_SIZE] = "";

	// Save the name of the program
	strncpy(progName, argv[0], BUF_TMP_SIZE - 1);

	if (argc != 2)				// One argument required
		help(progName);

	// Get the action option
	while (1)
	{
		rCode = getopt(argc, argv, "rsv");

		if (rCode == -1)
			break;

		switch (rCode)
		{
		case 's':				// Suspend action
			signalToSend = SIGUSR1;
			break;

		case 'r':				// Resume action
			signalToSend = SIGUSR2;
			break;

		case 'v':				// Version action
			version(progName);
			break;

		case ':':				// Wrong option
			help(progName);
			break;

		case '?':				// Wrong option
			help(progName);
			break;
		}
	}

	// First, we must know the autolock daemon pid
	// which is stored in the autolock pid file.
	//////////////////////////////////////////////

	// Open the autolock pid file
	fd = fopen(PID_AUTOLOCK_FILE, "r");

	if (fd == NULL)
	{
		fprintf(stderr,
			"UNABLE to open the autolock pid file (%s): %s\n",
			PID_AUTOLOCK_FILE, strerror(errno));

		return (EXIT_FAILURE);
	}

	// If the size of the autolock pid file is 0, the autolock
	// doesn't run.
	rCode = stat(PID_AUTOLOCK_FILE, &statbuf);

	if (rCode == -1)
	{
		fprintf(stderr,
			"UNABLE to stat the autolock pid file (%s): %s\n",
			PID_AUTOLOCK_FILE, strerror(errno));
		fclose(fd);
		return (EXIT_FAILURE);
	}
	else if (statbuf.st_size == 0)
	{
		fprintf(stderr,
			"ERROR: the autolock daemon seems no to be running\n");
		fclose(fd);
		return (EXIT_FAILURE);
	}

	// The autolock pid is the first bytes of the file
	nbReaded = fscanf(fd, "%d", &pid);

	if (nbReaded == 0)
	{
		fprintf(stderr,
			"UNABLE to read the autolock pid in the autolock pid file (%s): %s\n",
			PID_AUTOLOCK_FILE, strerror(errno));

		return (EXIT_FAILURE);
	}

	fclose(fd);

	// Next, we have to send the appropriate signal to the daemon
	/////////////////////////////////////////////////////////////

	rCode = kill(pid, signalToSend);

	if (rCode == -1)
	{
		fprintf(stderr, "UNABLE to send signals to the autolock daemon: %s\n",
			strerror(errno));
		exit(EXIT_FAILURE);
	}

	exit(EXIT_SUCCESS);
}

void help(char *progName)
{
	// Print the program usage informations
	///////////////////////////////////////

	printf("\n");
	printf("%s action \n", progName);
	printf("\n");
	printf(" action : \n");
	printf("   -s: suspend the autolock daemon\n");
	printf("   -r: resume the suspended autolock daemon\n");
	printf("   -v: print the version of this program\n");
	printf("\n");

	exit(EXIT_FAILURE);
}

void version(char *progName)
{
	// Print the program version
	////////////////////////////

	printf("%s " VERSION " by Vincent Guerin\n", progName);
	printf("  Usefull to suspend or resume the autolock daemon\n");
	printf("    <vincent.guerin@gemplus.com>, 27/10/2000\n");

	exit(EXIT_SUCCESS);
}
